from ujenkins.endpoints.builds import Builds
from ujenkins.endpoints.jobs import Jobs
from ujenkins.endpoints.nodes import Nodes
from ujenkins.endpoints.system import System
from ujenkins.endpoints.views import Views

__all__ = (
    'Builds',
    'Jobs',
    'Nodes',
    'System',
    'Views',
)
