from _typeshed import Incomplete
from typing import List, Optional, Tuple
from ujenkins.exceptions import JenkinsError as JenkinsError

JOB_BUILD_URL_RE: Incomplete

def construct_job_config(*, description: Optional[str] = ..., parameters: Optional[List[dict]] = ..., commands: Optional[List[str]] = ...) -> str: ...
def construct_node_config(*, name: str, remote_fs: Optional[str] = ..., executors: Optional[int] = ...) -> dict: ...
def parse_build_url(build_url: str) -> Tuple[str, int]: ...
